;*****************************************************************************
; ファイル名		CTC_INT.asm
; 対象マイコン 		Z80
; ファイル内容		Z80-CTC制御ソフト
;*****************************************************************************
;=============================================================================
;シンボル定義
;=============================================================================
PORTA	.EQU	0x80	;8255 ポートAアドレス
PORTB   .EQU	0x81	;8255 ポートBアドレス
PORTC	.EQU	0x82	;8255 ポートCアドレス
CWR		.EQU	0x83	;8255 コントロールアドレス
CW		.EQU	0x89	;コントロールワード
						;A,Bポート = OUTPUT
						;Cポート = INPUT

CTC0	.EQU	0x90	;CTCチャンネル0アドレス
CTC1	.EQU	0x91	;CTCチャンネル1アドレス
CTC2	.EQU	0x92	;CTCチャンネル2アドレス
CTC3	.EQU	0x93	;CTCチャンネル3アドレス

STACK	.EQU	0xC000	;スタックポインター

;======================================
;変数ポインタ
;======================================
CNT		.EQU	0x8000	;タイマーカウンタ

;=============================================================================
;初期化処理
;=============================================================================
			;======================================
			;コード領域アドレス指定
			;======================================
			.area	CODE(ABS)
			.org	0x0000

			;======================================
			;8255コントロールレジスタ設定
			;======================================
			LD		A,CW
			OUT		(CWR),A

			;======================================
			;CTCレジスタ設定
			;======================================
			;ベクタアドレスセット
			LD		A,0x08		;割り込みベクタアドレス下位をAレジスタにセットする
			OUT 	(CTC0),A	;割り込みベクタアドレス下位をセット

			;チャンネル3設定
			;CTC のチャンネル3のみタイマー割り込み設定
			;CPU の動作クロックを4MHz(250ns周期) としています
			;このクロックはCTCの1/16のプリスケーラを通り
			;ダウンカウンタ用定数の250をダウンカウントして
			;最終的に割り込み周期である1[ms] を作り出します
			LD		A,0b10000111	;CTC TIMER MODE
			OUT		(CTC3),A
			LD		A,250			;250ns * 16 * 250
			OUT		(CTC3),A		; = 1[ms]

			;======================================
			;Z80割り込み設定
			;======================================
			LD		A,0x01
			LD		I,A			;割り込みベクタアドレス上位をIレジスタにセット
			IM		2			;割り込みモード2を設定
			EI					;割り込み許可命令

			;======================================
			;変数初期値設定
			;======================================
			LD		B,0x00		;Bレジスタに0をセット
			LD		C,0x00		;Cレジスタに0をセット
			LD		D,0x00		;Dレジスタに0をセット

			;======================================
			;スタックポインタ設定
			;======================================
			LD		SP,STACK

			;======================================
			;Main関数へジャンプする
			;======================================
			JP		MAIN
			
;=============================================================================
;割り込みベクタテーブル
;=============================================================================
			;======================================
			;コード領域アドレス指定(割り込みベクタ指定)
			;======================================
			.area	CODE(ABS)
			.org	0x0108

			;======================================
			;割り込みベクタテーブル
			;======================================
			.DW		INTCTC0
			.DW		INTCTC1
			.DW		INTCTC2
			.DW		INTCTC3

;*****************************************************************************
;Mainルーチン
;*****************************************************************************
MAIN:
LOOP250m:	
			LD		A,B
			CP		250
			JP      NZ,LOOP250m		;真ならLOOP250mへ
			JR		LOOP1000m

LOOP1000m:
			LD		B,0
			INC		C
			LD		A,C
			CP		4
			JP      NZ,LOOP250m		;真ならLOOP1000mへ

LED:
			LD		C,0
			INC		D
			LD		A,D
			OUT		(PORTA),A
			OUT		(PORTB),A
			;LD		(0x8000),0x00
			JR		MAIN

;*****************************************************************************
;CTC割り込み(1ms)
;*****************************************************************************
INTCTC0:
INTCTC1:
INTCTC2:
INTCTC3:
			PUSH	AF
			INC		B
INT_END:
			POP		AF
			EI	
			RETI